@@ -76,10 +76,10 @@ def add_branch(tree,low,high,threshold_exponent=1.5,threshold_adjuster=0.75,
             points = angle_constraint(points,terminal,distal,0.75,False)
             if len(points) == 0:
                 continue
-            points = angle_constraint(points,terminal,proximal,0,False)
+            points = angle_constraint(points,terminal,proximal,0.2,False)
             if len(points) == 0:
                 continue
-            points = angle_constraint(points,distal,proximal,0,False)
+            points = angle_constraint(points,distal,proximal,0.2,False)
             if len(points) == 0:
                 continue
             if tree.data[vessel,17] >= 0:
