@@ -4,6 +4,8 @@ import matplotlib.pyplot as plt
 from matplotlib.patches import Circle
 from mpl_toolkits.mplot3d import Axes3D, proj3d, art3d
 from mpl_toolkits.mplot3d.art3d import Line3DCollection
+import vtk
+from scipy import optimize
 
 from ..branch_addition.basis import tangent_basis
 
@@ -387,6 +389,31 @@ def contour_collision(x1,y1,z1,nx1,ny1,nz1,r1,x2,y2,z2,nx2,ny2,nz2,r2,radius_buf
     if radius_buffer is not None:
         r1 += r1 + r1*radius_buffer
         r2 += r2 + r2*radius_buffer
+    contour_1 = vtk.vtkRegularPolygonSource()
+    contour_2 = vtk.vtkRegularPolygonSource()
+    contour_1.SetCenter([x1,y1,z1])
+    contour_2.SetCenter([x2,y2,z2])
+    contour_1.SetRadius(r1)
+    contour_2.SetRadius(r2)
+    contour_1.SetNormal([nx1,ny1,nz1])
+    contour_2.SetNormal([nx2,ny2,nz2])
+    contour_1.SetNumberOfSides(4)
+    contour_2.SetNumberOfSides(4)
+    collision = vtk.vtkCollisionDetectionFilter()
+    collision.SetInputConnection(0,contour_1.GetOutputPort())
+    collision.SetInputConnection(1,contour_2.GetOutputPort())
+    transform = vtk.vtkTransform()
+    matrix = vtk.vtkMatrix4x4()
+    collision.SetTransform(0,transform)
+    collision.SetMatrix(1,matrix)
+    collision.SetCollisionModeToAllContacts()
+    collision.GenerateScalarsOn()
+    collision.Update()
+    if collision.GetNumberOfContacts() > 0:
+        return True
+    else:
+        return False
+    """
     #print('{},{},{}'.format(x1,y1,z1))
     p1 = Point3D(Float(x1),Float(y1),Float(z1))
     p2 = Point3D(Float(x2),Float(y2),Float(z2))
@@ -406,47 +433,10 @@ def contour_collision(x1,y1,z1,nx1,ny1,nz1,r1,x2,y2,z2,nx2,ny2,nz2,r2,radius_buf
     #print('dist1: {} r1: {}'.format(dist1,r1))
     #print('dist2: {} r2: {}'.format(dist2,r2))
     if dist1 < r1 or dist2 < r2:
-        """
-        fig = plt.figure()
-        x,y,z,_,_,_,_ = splev(n,contours[0])
-        patch0 = Circle((0,0),r1,facecolor='y')
-        patch1 = Circle((0,0),r2,facecolor='r')
-        patch00 = Circle((0,0),r1+r1*radius_buffer,facecolor='y',alpha=0.2)
-        patch11 = Circle((0,0),r2+r2*radius_buffer,facecolor='r',alpha=0.2)
-        ax = fig.add_subplot(111,projection='3d')
-        ax.plot3D(x,y,z,c='b')
-        ax.scatter3D(x1,y1,z1)
-        ax.scatter3D(x2,y2,z2)
-        ax.plot3D(line_1[:,0],line_1[:,1],line_1[:,2],c='y')
-        ax.plot3D(line_2[:,0],line_2[:,1],line_2[:,2],c='r')
-        ax.plot3D(n1_vec[:,0],n1_vec[:,1],n1_vec[:,2],c='black')
-        ax.plot3D(n2_vec[:,0],n2_vec[:,1],n2_vec[:,2],c='black')
-        ax.set_xlim([min(x1,x2)-0.1,max(x1,x2)+0.1])
-        ax.set_ylim([min(y1,y2)-0.1,max(y1,y2)+0.1])
-        ax.set_zlim([min(z1,z2)-0.1,max(z1,z2)+0.1])
-        ax.add_patch(patch0)
-        pathpatch_2d_to_3d(patch0,z=0,normal=np.array([nx1,ny1,nz1]))
-        #print(dir(patch0))
-        #difs1 = (x1,y1,z1)-patch0._segment3d[0]
-        pathpatch_translate(patch0,(x1,y1,z1))
-        ax.add_patch(patch1)
-        #difs2 = (x2,y2,z2)-patch1._segment3d[0]
-        pathpatch_2d_to_3d(patch1,z=0,normal=np.array([nx2,ny2,nz2]))
-        pathpatch_translate(patch1,(x2,y2,z2))
-        ax.add_patch(patch00)
-        #difs1 = (x1,y1,z1)-patch00._segment3d[0]
-        pathpatch_2d_to_3d(patch00,z=0,normal=np.array([nx1,ny1,nz1]))
-        pathpatch_translate(patch00,(x1,y1,z1))
-        ax.add_patch(patch11)
-        #difs2 = (x2,y2,z2)-patch11._segment3d[0]
-        pathpatch_2d_to_3d(patch11,z=0,normal=np.array([nx2,ny2,nz2]))
-        pathpatch_translate(patch11,(x2,y2,z2))
-        plt.show()
-        """
         return True
     else:
         return False
-
+    """
 def contour_check(contours,t0,t1,radius_buffer):
     tl  = (t1 - t0)/10
     x1,y1,z1,r1,_,_,_   = splev(t0,contours[0])
@@ -475,156 +465,60 @@ def contour_check(contours,t0,t1,radius_buffer):
         return False
     elif np.isclose(nx1,0) and np.isclose(ny1,0):
         collision = contour_collision(x2,y2,z2,nx2,ny2,nz2,r2,x1,y1,z1,nx1,ny1,nz1,r1,radius_buffer,contours,t0,t1)
-        print(collision)
+        #print(collision)
         return collision
     else:
         collision = contour_collision(x1,y1,z1,nx1,ny1,nz1,r1,x2,y2,z2,nx2,ny2,nz2,r2,radius_buffer,contours,t0,t1)
-        print(collision)
+        #print(collision)
         return collision
 
 def contour_check_all(interps,radius_buffer):
     sample_t = []
     n = np.linspace(0,1,200)
+    total_collisions = 0
     for contours in interps:
-        #fig = plt.figure()
-        #ax = fig.add_subplot(111,projection='3d')
         tmp_t  = []
         t_list = []
-        t_list = np.linspace(0,1,2*len(contours[1]))
-        #x,y,z,r,_,_,_ = splev(t_list,contours[0])
-        #nx,ny,nz,_,_,_,_ = splev(t_list,contours[0],der=1)
-        #patches = []
-        #ax.scatter3D(x,y,z)
-        #ax.plot3D(x,y,z)
-        #for i in range(len(t_list)):
-        #    tmp_patch = Circle((0,0),r[i],facecolor='y')
-        #    tmp_patch2 = Circle((0,0),r[i],facecolor='r')
-        #    ax.add_patch(tmp_patch)
-        #    ax.add_patch(tmp_patch2)
-        #    pathpatch_2d_to_3d(tmp_patch2,z=0,normal=np.array([nx[i],ny[i],nz[i]]))
-        #    if i < len(t_list) - 1:
-        #        #pathpatch_2d_to_3d(tmp_patch2,z=0,normal=np.array([nx[i],ny[i],nz[i]]))
-        #        pathpatch_2d_to_3d(tmp_patch,z=0,normal=np.array([x[i+1]-x[i],y[i+1]-y[i],z[i+1]-z[i]]))
-        #    else:
-        #        pathpatch_2d_to_3d(tmp_patch,z=0,normal=np.array([x[i]-x[i-1],y[i]-y[i-1],z[i]-z[i-1]]))
-        #    pathpatch_translate(tmp_patch,(x[i],y[i],z[i]))
-        #    pathpatch_translate(tmp_patch2,(x[i],y[i],z[i]))
-        #plt.show()
-        """
-        nx,ny,nz,_,_,_,_ = splev(n,contours[0],der=1)
-        nxx,nyy,nzz,_,_,_,_ = splev(n,contours[0],der=2)
-        idx_x = np.argwhere(np.diff(np.sign(nx))!=0).flatten()
-        idx_y = np.argwhere(np.diff(np.sign(ny))!=0).flatten()
-        idx_z = np.argwhere(np.diff(np.sign(nz))!=0).flatten()
-        idx_xx = np.argwhere(np.diff(np.sign(nxx))!=0).flatten()
-        idx_yy = np.argwhere(np.diff(np.sign(nyy))!=0).flatten()
-        idx_zz = np.argwhere(np.diff(np.sign(nzz))!=0).flatten()
-        t_list.extend([i for i in idx_x if i not in t_list])
-        t_list.extend([i for i in idx_y if i not in t_list])
-        t_list.extend([i for i in idx_z if i not in t_list])
-        t_list.extend([i for i in idx_xx if i not in t_list])
-        t_list.extend([i for i in idx_yy if i not in t_list])
-        t_list.extend([i for i in idx_zz if i not in t_list])
-        if contours[0][-1] >= 3:
-            nxxx,nyyy,nzzz,_,_,_,_ = splev(n,contours[0],der=3)
-            idx_xxx = np.argwhere(np.diff(np.sign(nxxx))!=0).flatten()
-            idx_yyy = np.argwhere(np.diff(np.sign(nyyy))!=0).flatten()
-            idx_zzz = np.argwhere(np.diff(np.sign(nzzz))!=0).flatten()
-            t_list.extend([i for i in idx_xxx if i not in t_list])
-            t_list.extend([i for i in idx_yyy if i not in t_list])
-            t_list.extend([i for i in idx_zzz if i not in t_list])
-        t_list.insert(0,0)
-        t_list.append(len(n)-1)
-        t_list = n[sorted(t_list)]
-        print(t_list)
-        """
-        start = None
-        previous = False
-        count = 0
-        for t in t_list:
-            if t == 0:
-                tmp_t.append(t)
-                continue
-            t0 = tmp_t[-1]
-            t1 = t
-            collision = contour_check(contours,t0,t1,radius_buffer)
-            if not collision and not previous:
-                tmp_t.append(t)
-            elif not collision and previous:
-                print('resampling')
-                t_resample = np.linspace(start,t1,4*count)
-                for tt in t_resample:
-                    collision = contour_check(contours,t0,tt,radius_buffer)
-                    if not collision:
-                        print('resample accepted')
-                        tmp_t.append(tt)
-                        t0 = tmp_t[-1]
-                    else:
-                        continue
-                #m_collision = contour_check(contours,t0,t_m,radius_buffer)
-                #if not m_collision:
-                #    print('mid accepted')
-                #    tmp_t.append(t_m)
-                #else:
-                #    print('mid rejected')
-                #    tmp_t.append(t1)
-                previous = False
-                start = None
-                count = 0
-            elif collision and previous:
-                count += 1
-                continue
-            else:
-                #fig = plt.figure()
-                x0,y0,z0,r0,_,_,_ = splev(t0,contours[0])
-                nx0,ny0,nz0,_,_,_,_ = splev(t0,contours[0],der=1)
-                #mag0 = np.linalg.norm([nx0,ny0,nz0])
-                #nx0 = -mag0/nx0
-                #ny0 = -mag0/ny0
-                #nz0 = -mag0/nz0
-                #mag0 = np.linalg.norm([nx0,ny0,nz0])
-                x1,y1,z1,r1,_,_,_ = splev(t1,contours[0])
-                nx1,ny1,nz1,_,_,_,_ = splev(t1,contours[0],der=1)
-                #mag1 = np.linalg.norm([nx1,ny1,nz1])
-                #nx1 = -mag1/nx1
-                #ny1 = -mag1/ny1
-                #nz1 = -mag1/nz1
-                """
-                patch0 = Circle((0,0),r0,facecolor='y')
-                patch1 = Circle((0,0),r1,facecolor='r')
-                patch00 = Circle((0,0),r0+r0*radius_buffer,facecolor='y',alpha=0.2)
-                patch11 = Circle((0,0),r1+r1*radius_buffer,facecolor='r',alpha=0.2)
-                ax = fig.add_subplot(111,projection='3d')
-                ax.scatter3D(x0,y0,z0)
-                ax.scatter3D(x1,y1,z1)
-                ax.set_xlim([min(x0,x1)-0.1,max(x0,x1)+0.1])
-                ax.set_ylim([min(y0,y1)-0.1,max(y0,y1)+0.1])
-                ax.set_zlim([min(z0,z1)-0.1,max(z0,z1)+0.1])
-                ax.add_patch(patch0)
-                pathpatch_2d_to_3d(patch0,z=0,normal=np.array([nx0,ny0,nz0]))
-                pathpatch_translate(patch0,(x0,y0,z0))
-                ax.add_patch(patch1)
-                pathpatch_2d_to_3d(patch1,z=0,normal=np.array([nx1,ny1,nz1]))
-                pathpatch_translate(patch1,(x1,y1,z1))
-                ax.add_patch(patch00)
-                pathpatch_2d_to_3d(patch00,z=0,normal=np.array([nx0,ny0,nz0]))
-                pathpatch_translate(patch00,(x0,y0,z0))
-                ax.add_patch(patch11)
-                pathpatch_2d_to_3d(patch11,z=0,normal=np.array([nx1,ny1,nz1]))
-                pathpatch_translate(patch11,(x1,y1,z1))
-                plt.show()
-                """
-                if len(tmp_t) == 1:
-                    continue
-                else:
-                    start = tmp_t.pop(-1)
-                    previous = True
-                    count += 1
+        t_list = np.linspace(0,1,4*len(contours[1]))
+        collision_contours = set([])
+        collision_free = set([])
+        checking = set(list(range(1,len(t_list)-1)))
+        #t_op,f_op = optimize_contour(t_range[first],t_range[second],contours)
+        for idx in checking:
+            collide = False
+            for jdx in checking.difference(set([idx])):
+                if contour_check(contours,t_list[idx],t_list[jdx],radius_buffer):
+                    collision_contours.add(idx)
+                    collision_contours.add(jdx)
+                    collide = True
+            if not collide:
+                collision_free.add(idx)
+            #checking = checking.difference(collision_contours)
+            checking = checking.difference(collision_free)
+        collision_free = list(collision_free)
+        collision_free.sort()
+        if len(collision_contours) > 0:
+            pass
+            #plot_path_contours(contours,t_list,highlight=list(collision_contours))
+        for idx in range(len(collision_free)-1):
+            tmp_t.append(t_list[collision_free[idx]])
+            if collision_free[idx+1] - collision_free[idx]>1:
+                t_op,f_op = optimize_contour(t_list[collision_free[idx]],t_list[collision_free[idx+1]],contours,radius_buffer)
+                #if not contour_check(contours,t_list[collision_free[idx]],t_op,radius_buffer):
+                #    if not contour_check(contours,t_list[collision_free[idx+1]],t_op,radius_buffer):
+                #        tmp_t.append(t_op)
+                if f_op < 0:
+                    tmp_t.append(t_op)
+        tmp_t.append(t_list[collision_free[-1]])
+        if len(collision_contours) > 0:
+            pass
+            #plot_path_contours(contours,tmp_t)
         if len(tmp_t) == 1:
             tmp_t.append(1)
         else:
             tmp_t[-1] = 1
         sample_t.append(tmp_t)
+    print('Total Collisions Resolved: '+str(total_collisions))
     return sample_t
 
 def sv_data(interps,radius_buffer=0):
@@ -680,3 +574,87 @@ def plot_branches(data,branches):
                       [data[idx,2],data[idx,5]],label=str(i))
     plt.legend()
     plt.show()
+
+def plot_path_contours(interp,t_list,highlight=[]):
+    disks = []
+    lines = []
+    points = vtk.vtkPoints()
+    pts = []
+    radii = []
+    normals = []
+    actors = []
+    colors = vtk.vtkNamedColors()
+    for t in t_list:
+        x,y,z,r,_,_,_ = splev(t,interp[0])
+        nx,ny,nz,_,_,_,_ = splev(t,interp[0],der=1)
+        points.InsertNextPoint([x,y,z])
+        radii.append(r)
+        normals.append([nx,ny,nz])
+        pts.append([x,y,z])
+    polyline = vtk.vtkPolyLine()
+    polyline.GetPointIds().SetNumberOfIds(len(t_list))
+    for i in range(len(t_list)):
+        polyline.GetPointIds().SetId(i,i)
+    cells = vtk.vtkCellArray()
+    cells.InsertNextCell(polyline)
+    polydata = vtk.vtkPolyData()
+    polydata.SetPoints(points)
+    polydata.SetLines(cells)
+    mapper = vtk.vtkPolyDataMapper()
+    actor = vtk.vtkActor()
+    mapper.SetInputDataObject(polydata)
+    actor.SetMapper(mapper)
+    actor.GetProperty().SetColor(colors.GetColor3d('red'))
+    actors.append(actor)
+    for r in range(len(radii)):
+        tmp_disk = vtk.vtkRegularPolygonSource()
+        tmp_disk.SetRadius(radii[r])
+        tmp_disk.SetCenter(pts[r])
+        tmp_disk.SetNormal(normals[r])
+        tmp_disk.SetNumberOfSides(100)
+        mapper = vtk.vtkPolyDataMapper()
+        actor = vtk.vtkActor()
+        mapper.SetInputConnection(tmp_disk.GetOutputPort())
+        actor.SetMapper(mapper)
+        if r in highlight:
+            actor.GetProperty().SetColor(colors.GetColor3d('blue'))
+        else:
+            actor.GetProperty().SetColor(colors.GetColor3d('red'))
+        disks.append(tmp_disk)
+        actors.append(actor)
+    renderer = vtk.vtkRenderer()
+    renderer.SetBackground(colors.GetColor3d('white'))
+
+    render_window = vtk.vtkRenderWindow()
+    render_window.AddRenderer(renderer)
+    render_window.SetWindowName('Pathline with Contours')
+
+    interactor = vtk.vtkRenderWindowInteractor()
+    interactor.SetRenderWindow(render_window)
+    for actor in actors:
+        renderer.AddActor(actor)
+    render_window.Render()
+    interactor.Start()
+
+def optimize_contour(t0,t1,interp,radius_buffer):
+    nx0,ny0,nz0,_,_,_,_ = splev(t0,interp[0],der=1)
+    nx1,ny1,nz1,_,_,_,_ = splev(t1,interp[0],der=1)
+    n0 = np.array([nx0,ny0,nz0])
+    n1 = np.array([nx1,ny1,nz1])
+    n0 = n0/np.linalg.norm(n0) #.reshape(-1,1)
+    n1 = n1/np.linalg.norm(n1) #.reshape(-1,1)
+    def func(t,interp=interp,n0=n0,n1=n1,t0=t0,t1=t1):
+        nx,ny,nz,_,_,_,_ = splev(t,interp[0],der=1)
+        n = np.array([nx,ny,nz])
+        n = n/np.linalg.norm(n) #.reshape(-1,1)
+        #print(-np.dot(n0,n)*np.dot(n1,n))
+        a = np.dot(n0,n)
+        b = np.dot(n1,n)
+        c = 0
+        if contour_check(interp,t0,t,radius_buffer=radius_buffer):
+            c = 100
+        if contour_check(interp,t1,t,radius_buffer=radius_buffer):
+            c = 100
+        return -(a*b + a + b) + c
+    result = optimize.minimize_scalar(func,bounds=(t0,t1),method='bounded')
+    return result.x,result.fun
