@@ -92,8 +92,8 @@ terminating = False
 print('Union: {{}}\\nBad Edges: {{}}'.format(1,fe.GetOutput().GetNumberOfCells()))
 for i in range(2,len(better)):
     model_1.set_surface(tmp)
-    model_1.compute_boundary_faces(angle=45)
-    tmp = mesh_utils.remesh(model_1.get_polydata(),hmin=0.02,hmax=0.02)
+    #model_1.compute_boundary_faces(angle=45)
+    tmp = mesh_utils.remesh(model_1.get_polydata(),hmin=0.01,hmax=0.02)
     model_1.set_surface(tmp)
     model_2.set_surface(better[i])
     print('models created')
@@ -121,6 +121,7 @@ for i in range(2,len(better)):
         #strip.Update()
         tri.SetInputDataObject(fill.GetOutput())
         tri.Update()
+        clean2.SetInputDataObject(tri.GetOutput())
         clean2.SetConvertLinesToPoints(1)
         clean2.SetConvertPolysToLines(1)
         clean2.SetConvertStripsToPolys(1)
@@ -135,6 +136,22 @@ for i in range(2,len(better)):
             terminating = True
         else:
             terminating = False
+    #colors = vtk.vtkNamedColors()
+    #actor = vtk.vtkActor()
+    #mapper = vtk.vtkPolyDataMapper()
+    #mapper.SetInputDataObject(tmp)
+    #actor.SetMapper(mapper)
+    #actor.GetProperty().SetColor(colors.GetColor3d('red'))
+    #renderer = vtk.vtkRenderer()
+    #renderer.SetBackground(colors.GetColor3d('white'))
+    #render_window = vtk.vtkRenderWindow()
+    #render_window.AddRenderer(renderer)
+    #render_window.SetWindowName('Polydata')
+    #interactor = vtk.vtkRenderWindowInteractor()
+    #interactor.SetRenderWindow(render_window)
+    #renderer.AddActor(actor)
+    #render_window.Render()
+    #interactor.Start()
 ############################
 # COMBINE FACES
 ############################
@@ -179,12 +196,14 @@ if not terminating:
     ###############################
     # LOCAL SMOOTHING (not included)
     ###############################
-    smoothing_params = {{'method':'constrained', 'num_iterations':5, 'constrain_factor':0.2, 'num_cg_solves':30}}
+    #smoothing_params = {{'method':'constrained', 'num_iterations':5, 'constrain_factor':0.2, 'num_cg_solves':30}}
     smooth_model = model.get_polydata()
     for idx, contour_set in enumerate(contour_list):
          if idx == 0:
               continue
+         smoothing_params = {{'method':'constrained', 'num_iterations':3, 'constrain_factor':0.1+(0.9*(1-contour_set[0].get_radius()/contour_list[0][0].get_radius())), 'num_cg_solves':30}}
          smooth_model = geometry.local_sphere_smooth(smooth_model,contour_set[0].get_radius()*2,contour_set[0].get_center(),smoothing_params)
          print('local sphere smoothing {{}}'.format(idx))
     model.set_surface(smooth_model)
+
 """
